import { Component } from '@serverless-devs/s-core';
import { DEFAULT } from './static';
const fse = require('fs-extra');
const path = require('path');

const {
  REGION,
  SERVICE,
  FUNCTION,
  TRIGGERS,
  DOMAINS,
  SRARTCONFIG
} = DEFAULT;

interface ProjectConifg {
  ProjectName: string;
}

interface CredentialsConfig {
  AccountID: string;
  AccessKeyID: string;
  AccessKeySecret: string;
}

interface TriggerConifg {
  Name: string;
  Type: string;
  Parameters: any;
  Domains?: [];
}

interface LogConfig {
  LogStore: string
  Project: string
}

interface StartContext {
  isReadCache?: boolean
  Port?: number
  Start: string
}

interface PropertiesConfig {
  Region?: string
  CodeUri: string
  Domains?: []
  Environment?: []
  Log?: LogConfig
  Detailed: any
}

interface InputsContext {
  Project: ProjectConifg
  Credentials: CredentialsConfig
  Properties: PropertiesConfig
}

class Framework extends Component {
  constructor(id?: string) {
    super(id);
  }

  async deploy(inputs: InputsContext ) {
    const fc = await this.load('fc', 'Component');
    const { Project, Credentials, Properties } = inputs;
    const {
      Region = REGION,
      CodeUri,
      Domains,
      Environment,
      Log,
      Detailed = {}
    } = Properties;
    if (!CodeUri) {
      throw new Error('CodeUri does not exist.');
    }
    let {
      Service,
      Function,
      StartConfig
    } = Detailed;

    if (!Service) {
      if (fc.state.Service) {
        Service = fc.state.Service;
      } else {
        Service = SERVICE;
      }
    }

    if (!Function) {
      if (fc.state.Function) {
        Function = fc.state.Function;
      } else {
        Function = FUNCTION;
      }
    }
    
    if (Log) {
      Service.Log = Log;
    }

    if (!Function.Triggers) {
      Function.Triggers = TRIGGERS;
    }

    Function.Triggers = this.getTriggers(Domains, Function.Triggers);

    if (Environment) {
      Function.Environment = Environment;
    }

    await this.handlerStartConfig(CodeUri, StartConfig);
    
    Function.CodeUri = CodeUri;

    await fc.deploy({
      Args: { type: 'all' },
      State: fc.state,

      Credentials,
      Project,
      Properties: {
        Region,
        Function,
        Service
      }
    });
    fc.state = { Function, Service };
    fc.save();
  }

  private async handlerStartConfig(codeUri: string, startConfig?: StartContext) {
    if (codeUri.endsWith('.zip') || codeUri.endsWith('.jar') || codeUri.endsWith('.war')) {
      return;
    }
    
    const endPath = path.resolve(process.cwd(), `${codeUri}/bootstrap`);
    const hasBootstrapFile = await fse.pathExists(endPath);

    if (hasBootstrapFile) {
      if (!startConfig || startConfig.isReadCache) {
        return;
      }
    }

    const { Port, Start } = startConfig || SRARTCONFIG;
    await fse.writeFile(endPath, `#!/usr/bin/env bash\n\nexport PORT=${Port}\n${Start}`, {
      mode: '0755'
    });
  }

  private getTriggers(domains: any, triggers: any) {
    const notDomain = !domains;
    return triggers.map((trigger: TriggerConifg) => {
      // 如果没有自定义域名 并且 trigger 有域名配置
      if (notDomain && trigger.Parameters.Domains && trigger.Parameters.Domains[0]) {
        return trigger;
      } 
      trigger.Parameters.Domains = [];
      if (notDomain) {
        domains = DOMAINS;
      }
      for (const domain of domains) {
        const qualifier = trigger.Parameters.Qualifier || 'LATEST';
        const routes = domain.Routes || [];
        for (const route of routes) {
          const q = route.Qualifier || 'LATEST';
          if (qualifier === q) {
            trigger.Parameters.Domains.push(domain);
            return trigger;
          }
        }
      }
      
      return trigger;
    });
  }
}

module.exports = Framework;